@@ -631,7 +631,8 @@ sub replycmd {
 
     if($sender && $self->opcode eq WEBZEPHYR_OPCODE) {
         $class = WEBZEPHYR_CLASS;
-        $instance = $self->sender;
+        $instance = $self->pretty_sender;
+        $instance =~ s/-webzephyr$//;
         $to = WEBZEPHYR_PRINCIPAL;
     } elsif($self->class eq WEBZEPHYR_CLASS
             && $self->is_loginout) {
@@ -662,10 +663,10 @@ sub replycmd {
     }
 
     if (lc $class ne 'message') {
-        $cmd .= " -c " . BarnOwl::quote($self->class);
+        $cmd .= " -c " . BarnOwl::quote($class);
     }
     if (lc $instance ne 'personal') {
-        $cmd .= " -i " . BarnOwl::quote($self->instance);
+        $cmd .= " -i " . BarnOwl::quote($instance);
     }
     if ($to ne '') {
         $to = strip_realm($to);
@@ -1047,7 +1048,11 @@ sub format_sender {
     my $m = shift;
     my $sender = $m->long_sender;
     $sender =~ s/\n.*$//s;
-    return "  (" . $sender . '@color[default]' . ")";
+    if (BarnOwl::getvar('colorztext') eq 'on') {
+      return "  (" . $sender . '@color[default]' . ")";
+    } else {
+      return "  ($sender)";
+    }
 }
 
 sub indent_body
@@ -1057,7 +1062,7 @@ sub indent_body
 
     my $body = $m->body;
     if ($m->{should_wordwrap}) {
-      $body = BarnOwl::wordwrap($body, BarnOwl::getnumcols()-8);
+      $body = BarnOwl::wordwrap($body, BarnOwl::getnumcols()-9);
     }
     # replace newline followed by anything with
     # newline plus four spaces and that thing.
@@ -1086,6 +1091,11 @@ BarnOwl::create_style("oneline", "BarnOwl::Style::OneLine");
 
 ################################################################################
 
+sub maybe {
+    my $thing = shift;
+    return defined($thing) ? $thing : "";
+}
+
 sub format_login {
   my $self = shift;
   my $m = shift;
@@ -1135,8 +1145,8 @@ sub format_chat
   else {
     $line = sprintf(BASE_FORMAT,
                     $dirsym,
-                    $m->context,
-                    $m->subcontext,
+                    maybe($m->context),
+                    maybe($m->subcontext),
                     ($dir eq 'out'
                      ? $m->pretty_recipient
                      : $m->pretty_sender));
